@@ -64,14 +64,14 @@ Manage Beaker clusters with :data:`Beaker.cluster`.
 For example, you can get information about a cluster with
 :meth:`Beaker.cluster.get() <services.ClusterClient.get>`:
 
->>> beaker.cluster.get(beaker_cluster_name).autoscale
+>>> beaker.cluster.get(beaker_cloud_cluster_name).autoscale
 True
 
 Or you could check how many GPUs are free on an on-premise cluster with
 :meth:`Beaker.cluster.utilization() <services.ClusterClient.utilization>`:
 
 >>> free_gpus = 0
->>> for node_util in beaker.cluster.utilization(beaker_on_prem_cluster_name):
+>>> for node_util in beaker.cluster.utilization(beaker_on_prem_cluster_name).nodes:
 ...     free_gpus += node_util.free.gpu_count
 
 Nodes
@@ -96,7 +96,7 @@ For example, upload a local Docker image to Beaker with
 >>> image = beaker.image.create(beaker_image_name, docker_image_name, quiet=True)
 <BLANKLINE>
 
-The object returned is the same :class:`~data_model.Image` object you get from
+The object returned is the same :class:`~data_model.image.Image` object you get from
 :meth:`Beaker.image.get() <services.ImageClient.get>`.
 It contains some metadata about the image:
 
@@ -130,16 +130,16 @@ For example, create an experiment with :meth:`Beaker.experiment.create() <servic
 ... )
 
 Wait for the experiment to complete with
-:meth:`Beaker.experiment.await_all() <services.ExperimentClient.await_all>`:
+:meth:`Beaker.experiment.wait_for() <services.ExperimentClient.wait_for>`:
 
->>> experiment = beaker.experiment.await_all(
+>>> experiment = beaker.experiment.wait_for(
 ...     experiment,
-...     timeout=60 * 3,
+...     timeout=60 * 5,
 ...     quiet=True,
 ... )[0]
-<BLANKLINE>
 
-Get the logs from the experiment with :meth:`Beaker.experiment.logs() <services.ExperimentClient.logs>`:
+Get the logs from the execution of a task in an experiment with
+:meth:`Beaker.experiment.logs() <services.ExperimentClient.logs>`:
 
 >>> logs = "".join([
 ...    line.decode() for line in
@@ -147,10 +147,10 @@ Get the logs from the experiment with :meth:`Beaker.experiment.logs() <services.
 ... ])
 <BLANKLINE>
 
-Get the results from all tasks in an experiment with
+Get the results from a task in an experiment with
 :meth:`Beaker.experiment.results <services.ExperimentClient.results>`:
 
->>> task, results = beaker.experiment.results(experiment)[0]
+>>> results = beaker.experiment.results(experiment)
 
 Jobs
 ----
@@ -159,7 +159,7 @@ Manage Beaker jobs with :data:`Beaker.job`.
 
 For example, get the logs from a job with :meth:`Beaker.job.logs() <services.JobClient.logs>`
 (equivalent to :meth:`Beaker.experiment.logs() <services.ExperimentClient.logs>` when there is
-only one job in the experiment):
+only one task in the experiment):
 
 >>> job = experiment.jobs[0]
 >>> logs = "".join([
@@ -202,6 +202,15 @@ For example, you can read, write, or delete secrets with
 :meth:`Beaker.secret.write() <services.SecretClient.write>`, and
 :meth:`Beaker.secret.delete() <services.SecretClient.delete>`, respectively.
 
+Groups
+------
+
+Manage Beaker groups with :data:`Beaker.group`.
+
+For example, create a group with :meth:`Beaker.group.create() <services.GroupClient.create>`:
+
+>>> group = beaker.group.create(group_name, experiment)
+
 """
 
 from .client import *
