@@ -26,11 +26,13 @@ class Dataset(BaseModel):
     id: str
     name: Optional[str] = None
     full_name: Optional[str] = None
+    description: Optional[str] = None
     owner: Account
     author: Account
     created: datetime
     committed: Optional[datetime] = None
     workspace_ref: WorkspaceRef
+    source_execution: Optional[str] = None
     storage: Optional[DatasetStorage] = None
 
     @property
@@ -81,3 +83,15 @@ class DatasetManifest(BaseModel):
 class DatasetsPage(BaseModel):
     data: Tuple[Dataset, ...]
     next_cursor: Optional[str] = None
+
+
+class DatasetSpec(BaseModel):
+    workspace: Optional[str] = None
+    description: Optional[str] = None
+    fileheap: Optional[bool] = None
+
+
+class DatasetPatch(BaseModel):
+    name: Optional[str] = None
+    description: Optional[str] = None
+    commit: Optional[bool] = None
