@@ -13,6 +13,7 @@ class Experiment(BaseModel):
     id: str
     name: Optional[str] = None
     full_name: Optional[str] = None
+    description: Optional[str] = None
     owner: Account
     author: Account
     created: datetime
@@ -48,3 +49,8 @@ class Task(BaseModel):
 class ExperimentsPage(BaseModel):
     data: Tuple[Experiment, ...]
     next_cursor: Optional[str] = None
+
+
+class ExperimentPatch(BaseModel):
+    name: Optional[str] = None
+    description: Optional[str] = None
