@@ -6,6 +6,7 @@ from typing import TYPE_CHECKING, Deque, Dict, Generator, Optional, Tuple, Union
 from ..aliases import PathOrStr
 from ..data_model import *
 from ..exceptions import *
+from ..util import path_is_relative_to
 from .service_client import ServiceClient
 
 if TYPE_CHECKING:
@@ -64,10 +65,12 @@ class DatasetClient(ServiceClient):
         *sources: PathOrStr,
         target: Optional[PathOrStr] = None,
         workspace: Optional[str] = None,
+        description: Optional[str] = None,
         force: bool = False,
-        max_workers: int = 8,
+        max_workers: Optional[int] = None,
         quiet: bool = False,
         commit: bool = True,
+        strip_paths: bool = False,
     ) -> Dataset:
         """
         Create a dataset with the source file(s).
@@ -78,10 +81,19 @@ class DatasetClient(ServiceClient):
             a directory of this name.
         :param workspace: The workspace to upload the dataset to. If not specified,
             :data:`Beaker.config.default_workspace <beaker.Config.default_workspace>` is used.
+        :param description: Text description for the dataset.
         :param force: If ``True`` and a dataset by the given name already exists, it will be overwritten.
         :param max_workers: The maximum number of thread pool workers to use to upload files concurrently.
         :param quiet: If ``True``, progress won't be displayed.
         :param commit: Whether to commit the dataset after successful upload.
+        :param strip_paths: If ``True``, all source files and directories will be uploaded under their name,
+            not their path. E.g. the file "docs/source/index.rst" would be uploaded as just "index.rst",
+            instead of "docs/source/index.rst".
+
+            .. note::
+                This only applies to source paths that are children of the current working directory.
+                If a source path is outside of the current working directory, it will always
+                be uploaded under its name only.
 
         :raises ValueError: If the name is invalid.
         :raises DatasetConflict: If a dataset by that name already exists and ``force=False``.
@@ -94,7 +106,7 @@ class DatasetClient(ServiceClient):
         :raises HTTPError: Any other HTTP exception that can occur.
         """
         self.validate_beaker_name(name)
-        workspace: Workspace = self.resolve_workspace(workspace)
+        workspace_id = self.resolve_workspace(workspace).id
 
         # Create the dataset.
         def make_dataset() -> Dataset:
@@ -103,10 +115,9 @@ class DatasetClient(ServiceClient):
                     "datasets",
                     method="POST",
                     query={"name": name},
-                    data={
-                        "workspace": workspace.id,  # type: ignore
-                        "fileheap": True,
-                    },
+                    data=DatasetSpec(
+                        workspace=workspace_id, description=description, fileheap=True
+                    ),
                     exceptions_for_status={409: DatasetConflict(name)},
                 ).json()
             )
@@ -122,7 +133,15 @@ class DatasetClient(ServiceClient):
         assert dataset_info.storage is not None
 
         # Upload the file(s).
-        self.sync(dataset_info, *sources, target=target, quiet=quiet, max_workers=max_workers)
+        if sources:
+            self.sync(
+                dataset_info,
+                *sources,
+                target=target,
+                quiet=quiet,
+                max_workers=max_workers,
+                strip_paths=strip_paths,
+            )
 
         # Commit the dataset.
         if commit:
@@ -146,7 +165,7 @@ class DatasetClient(ServiceClient):
             self.request(
                 f"datasets/{self.url_quote(dataset_id)}",
                 method="PATCH",
-                data={"commit": True},
+                data=DatasetPatch(commit=True),
                 exceptions_for_status={404: DatasetNotFound(self._not_found_err_msg(dataset))},
             ).json()
         )
@@ -156,8 +175,9 @@ class DatasetClient(ServiceClient):
         dataset: Union[str, Dataset],
         target: Optional[PathOrStr] = None,
         force: bool = False,
-        max_workers: int = 8,
+        max_workers: Optional[int] = None,
         quiet: bool = False,
+        validate_checksum: bool = True,
     ):
         """
         Download a dataset.
@@ -167,11 +187,14 @@ class DatasetClient(ServiceClient):
         :param max_workers: The maximum number of thread pool workers to use to download files concurrently.
         :param force: If ``True``, existing local files will be overwritten.
         :param quiet: If ``True``, progress won't be displayed.
+        :param validate_checksum: If ``True``, the checksum of every file downloaded will be verified.
 
         :raises DatasetNotFound: If the dataset can't be found.
         :raises DatasetReadError: If the :data:`~beaker.data_model.dataset.Dataset.storage` hasn't been set.
         :raises FileExistsError: If ``force=False`` and an existing local file clashes with a file
             in the Beaker dataset.
+        :raises ChecksumFailedError: If ``validate_checksum=True`` and the digest of one of the
+            downloaded files doesn't match the expected digest.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
         :raises HTTPError: Any other HTTP exception that can occur.
         """
@@ -230,6 +253,7 @@ class DatasetClient(ServiceClient):
                         target_path,
                         progress,
                         bytes_task,
+                        validate_checksum,
                     )
                     download_futures.append(future)
 
@@ -246,6 +270,8 @@ class DatasetClient(ServiceClient):
         offset: int = 0,
         length: int = -1,
         max_retries: int = 5,
+        quiet: bool = False,
+        validate_checksum: bool = True,
     ) -> Generator[bytes, None, None]:
         """
         Stream download the contents of a single file from a dataset.
@@ -256,10 +282,14 @@ class DatasetClient(ServiceClient):
         :param length: Number of bytes to read.
         :param max_retries: Number of times to restart the download when HTTP errors occur.
             Errors can be expected for very large files.
+        :param quiet: If ``True``, progress won't be displayed.
+        :param validate_checksum: If ``True``, the checksum of the downloaded bytes will be verified.
 
         :raises DatasetNotFound: If the dataset can't be found.
         :raises DatasetReadError: If the :data:`~beaker.data_model.dataset.Dataset.storage` hasn't been set.
         :raises FileNotFoundError: If the file doesn't exist in the dataset.
+        :raises ChecksumFailedError: If ``validate_checksum=True`` and the digest of the downloaded
+            bytes don't match the expected digest.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
         :raises HTTPError: Any other HTTP exception that can occur.
         """
@@ -283,9 +313,20 @@ class DatasetClient(ServiceClient):
                 response.headers[self.HEADER_LAST_MODIFIED], "%a, %d %b %Y %H:%M:%S %Z"
             ),
         )
-        yield from self._stream_file(
-            dataset.storage, file_info, offset=offset, length=length, max_retries=max_retries
-        )
+        from ..progress import get_unsized_dataset_fetch_progress
+
+        with get_unsized_dataset_fetch_progress(quiet=quiet) as progress:
+            task_id = progress.add_task("Downloading", total=None)
+            for bytes_chunk in self._stream_file(
+                dataset.storage,
+                file_info,
+                offset=offset,
+                length=length,
+                max_retries=max_retries,
+                validate_checksum=validate_checksum,
+            ):
+                progress.update(task_id, advance=len(bytes_chunk))
+                yield bytes_chunk
 
     def delete(self, dataset: Union[str, Dataset]):
         """
@@ -310,7 +351,8 @@ class DatasetClient(ServiceClient):
         *sources: PathOrStr,
         target: Optional[PathOrStr] = None,
         quiet: bool = False,
-        max_workers: int = 8,
+        max_workers: Optional[int] = None,
+        strip_paths: bool = False,
     ) -> None:
         """
         Sync local files or directories to an uncommitted dataset.
@@ -321,6 +363,14 @@ class DatasetClient(ServiceClient):
             a directory of this name.
         :param max_workers: The maximum number of thread pool workers to use to upload files concurrently.
         :param quiet: If ``True``, progress won't be displayed.
+        :param strip_paths: If ``True``, all source files and directories will be uploaded under their name,
+            not their path. E.g. the file "docs/source/index.rst" would be uploaded as just "index.rst",
+            instead of "docs/source/index.rst".
+
+            .. note::
+                This only applies to source paths that are children of the current working directory.
+                If a source path is outside of the current working directory, it will always
+                be uploaded under its name only.
 
         :raises DatasetNotFound: If the dataset can't be found.
         :raises DatasetWriteError: If the dataset was already committed.
@@ -343,20 +393,21 @@ class DatasetClient(ServiceClient):
             path_info: Dict[Path, Tuple[Path, int]] = {}
             for name in sources:
                 source = Path(name)
+                strip_path = strip_paths or not path_is_relative_to(source, ".")
                 if source.is_file():
-                    target_path = Path(source.name)
+                    target_path = Path(source.name) if strip_path else source
                     if target is not None:
                         target_path = Path(target) / target_path
                     size = source.lstat().st_size
                     if size == 0:
-                        raise UnexpectedEOFError(str(source))
+                        raise UnexpectedEOFError(f"'{source}', empty files are not allowed")
                     path_info[source] = (target_path, size)
                     total_bytes += size
                 elif source.is_dir():
                     for path in source.glob("**/*"):
                         if path.is_dir():
                             continue
-                        target_path = path.relative_to(source)
+                        target_path = path.relative_to(source) if strip_path else path
                         if target is not None:
                             target_path = Path(target) / target_path
                         size = path.lstat().st_size
@@ -434,7 +485,7 @@ class DatasetClient(ServiceClient):
             total += file_info.size
         return total
 
-    def rename(self, dataset: Union[str, Dataset], new_name: str) -> Dataset:
+    def rename(self, dataset: Union[str, Dataset], name: str) -> Dataset:
         """
         Rename a dataset.
 
@@ -447,20 +498,31 @@ class DatasetClient(ServiceClient):
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
         :raises HTTPError: Any other HTTP exception that can occur.
         """
-        self.validate_beaker_name(new_name)
+        self.validate_beaker_name(name)
         dataset_id = self.resolve_dataset(dataset).id
         return Dataset.from_json(
             self.request(
                 f"datasets/{self.url_quote(dataset_id)}",
                 method="PATCH",
-                data={"name": new_name},
+                data=DatasetPatch(name=name),
                 exceptions_for_status={
-                    409: DatasetConflict(new_name),
+                    409: DatasetConflict(name),
                     404: DatasetNotFound(dataset_id),
                 },
             ).json()
         )
 
+    def url(self, dataset: Union[str, Dataset]) -> str:
+        """
+        Get the URL for a dataset.
+
+        :param dataset: The dataset ID, name, or object.
+
+        :raises DatasetNotFound: If the dataset can't be found.
+        """
+        dataset_id = self.resolve_dataset(dataset).id
+        return f"{self.config.agent_address}/ds/{self.url_quote(dataset_id)}/details"
+
     def _not_found_err_msg(self, dataset: Union[str, Dataset]) -> str:
         dataset = dataset if isinstance(dataset, str) else dataset.id
         return (
@@ -551,7 +613,10 @@ class DatasetClient(ServiceClient):
     def _decode_digest(self, encoded_digest: str) -> bytes:
         import base64
 
-        _, encoded = encoded_digest.split(" ", 2)
+        if self.SHA256 in encoded_digest:
+            _, encoded = encoded_digest.split(" ", 2)
+        else:
+            encoded = encoded_digest
         return base64.standard_b64decode(encoded)
 
     def _iter_files(self, storage: DatasetStorage) -> Generator[FileInfo, None, None]:
@@ -582,11 +647,14 @@ class DatasetClient(ServiceClient):
         self,
         storage: DatasetStorage,
         file_info: FileInfo,
-        chunk_size: int = 1,
+        chunk_size: int = 1024,
         offset: int = 0,
         length: int = -1,
         max_retries: int = 5,
+        validate_checksum: bool = True,
     ) -> Generator[bytes, None, None]:
+        import hashlib
+
         def stream_file(offset: int, length: int) -> Generator[bytes, None, None]:
             headers = {}
             if offset > 0 and length > 0:
@@ -605,11 +673,14 @@ class DatasetClient(ServiceClient):
             for chunk in response.iter_content(chunk_size=chunk_size):
                 yield chunk
 
+        sha256_hash = hashlib.sha256() if (offset == 0 and validate_checksum) else None
         retries = 0
         while True:
             try:
                 for chunk in stream_file(offset, length):
                     offset += len(chunk)
+                    if sha256_hash is not None:
+                        sha256_hash.update(chunk)
                     yield chunk
                 break
             except HTTPError:
@@ -617,6 +688,15 @@ class DatasetClient(ServiceClient):
                     raise
                 retries += 1
 
+        # Validate digest.
+        if sha256_hash is not None:
+            digest = sha256_hash.digest()
+            if self._decode_digest(file_info.digest) != digest:
+                raise ChecksumFailedError(
+                    f"Checksum for '{file_info.path}' failed. "
+                    f"Expected '{file_info.digest}', got '{self._encode_digest(digest)}'."
+                )
+
     def _download_file(
         self,
         storage: DatasetStorage,
@@ -624,6 +704,7 @@ class DatasetClient(ServiceClient):
         target_path: Path,
         progress: "Progress",
         task_id: "TaskID",
+        validate_checksum: bool = True,
     ) -> int:
         import tempfile
 
@@ -632,7 +713,7 @@ class DatasetClient(ServiceClient):
         target_dir.mkdir(exist_ok=True, parents=True)
         tmp_target = tempfile.NamedTemporaryFile("w+b", dir=target_dir, delete=False, suffix=".tmp")
         try:
-            for chunk in self._stream_file(storage, file_info):
+            for chunk in self._stream_file(storage, file_info, validate_checksum=validate_checksum):
                 total_bytes += len(chunk)
                 tmp_target.write(chunk)
                 progress.update(task_id, advance=len(chunk))
